import os.path
import uuid
import subprocess

import magic

from django.http import HttpResponse
from django.shortcuts import render
from django.utils import timezone

from whispermaphone import settings
from .models import Thought

# A dict mapping allowed mime types to file extensions
ALLOWED_MEDIA_TYPES = {
    "image/png": "png",
    "image/jpeg": "jpeg",
    "audio/x-m4a": "m4a",
    "audio/mp3": "mp3",
    "video/mp4": "mp4",
    "video/quicktime": "mov",
}


def index(request):
    authenticated = False
    try:
        if request.COOKIES["password"] == "ChromaticWave":
            authenticated = True
    except KeyError:
        pass

    try:
        highlighted_uuid = uuid.UUID(request.GET.get("show", ""))
    except ValueError:
        highlighted_uuid = ""

    thoughts = Thought.objects.order_by("-posted")

    for thought in thoughts:
        thought.timezone = timezone.get_fixed_timezone(-thought.timezone_offset)
        offset_hours = -thought.timezone_offset / 60
        if offset_hours == int(offset_hours):
            offset_hours = int(offset_hours)
        if offset_hours > 0:
            offset_hours = "+" + str(offset_hours)
        thought.offset_hours = offset_hours

    return render(request, "whispermaphone/index.html", {
        "thoughts": thoughts,
        "highlighted": highlighted_uuid,
        "authenticated": authenticated
    })


def post(request):
    try:
        if not request.COOKIES["password"] == "ChromaticWave":
            return render(request, "whispermaphone/login.html", status=401)
    except KeyError:
        return render(request, "whispermaphone/login.html", status=401)

    if request.method == "POST":
        if len(request.POST["text"].strip()) == 0:
            return HttpResponse("Need some text.", status=400)

        if len(request.POST["text"]) > 140:
            return HttpResponse("Content too long", status=400)

        thought = Thought(
            text=request.POST["text"],
            extended_text=request.POST["extended_text"],
            timezone_offset=request.POST["timezone_offset"]
        )

        if "media" in request.FILES:
            f = request.FILES["media"]

            # 16 MB file size limit.
            if f.size > 2**24:
                return HttpResponse("Content too long", status=400)

            chunk = next(f.chunks(chunk_size=2048))
            media_type = magic.from_buffer(chunk, mime="True")

            if media_type not in ALLOWED_MEDIA_TYPES:
                return HttpResponse("Invalid media type", status=400)

            f.name = f"{thought.uuid}.{ALLOWED_MEDIA_TYPES[media_type]}"

            thought.media = request.FILES["media"]

            if media_type == "audio/x-m4a":
                # This is a hack-fix because I want to be able to upload audio
                # In the future, this should be refactored to convert file types
                #   using ffmpeg.js on the client side (so there are 0 security concerns)
                #   and then the backend just has to check a 3 item whitelist
                thought.save()  # Save so that we have a file to work with
                subprocess.run(["ffmpeg",
                                "-i", thought.media.path,
                                "-codec:a", "aac",
                                os.path.join(settings.MEDIA_ROOT, f"{thought.uuid}.aac")
                                ])
                os.remove(f"{thought.uuid}.m4a")  # Remove the original file
                thought.media.name = f"{thought.uuid}.aac"  # Update the file in the DB

        thought.save()

    return render(request, "whispermaphone/post.html", {})


def about(request):
    return render(request, "whispermaphone/about.html")
